$releaseDefinition = "DeployWebApp"

# WebComponent and SqlComponent are components used in release definition 'DeployWebApp'. if you have more componenets in templated pass build number to those componenets in below property bag
# Example format:   ""ComponenetName:Build"" : ""BuildNumber"". 'Build' is keyword in format

$releaseName = $releaseDefinition + (Get-Date).DateTime
$deploymentPropertyBag = "{ 
                            ""WebComponent:Build"" : ""WebAppBuild_20140814.1"",
							""SqlComponent:Build"" : ""SqlBuild_20140815.1"",                            
                            ""ReleaseName"" : ""$releaseName""                             
                         }"

$propertyBag = [System.Uri]::EscapeDataString($deploymentPropertyBag)

$exitCode = 0

trap
{
  $e = $error[0].Exception
  $e.Message
  $e.StackTrace
  if ($exitCode -eq 0) { $exitCode = 1 }
}

$scriptName = $MyInvocation.MyCommand.Name
$scriptPath = Split-Path -Parent (Get-Variable MyInvocation -Scope Script).Value.MyCommand.Path

Push-Location $scriptPath  

$serverName = "server:1000"
$orchestratorService = "http://$serverName/account/releaseManagementService/_apis/releaseManagement/OrchestratorService"

$status = @{
    "2" = "InProgress";
    "3" = "Released";
    "4" = "Stopped";
    "5" = "Rejected";
    "6" = "Abandoned";
}

#For Update3 use api-version=2.0 for Update4 use api-version=3.0.
$uri = "$orchestratorService/InitiateRelease?releaseTemplateName=" + $releaseDefinition + "&deploymentPropertyBag=" + $propertyBag + "&api-version=3.0"
"Executing the following API call:`n`n$uri"

$wc = New-Object System.Net.WebClient
#$wc.UseDefaultCredentials = $true
# rmuser should be part rm users list and he should have permission to trigger the release.

$wc.Credentials = new-object System.Net.NetworkCredential("rmuser", "rmuserpassword", "rmuserdomain")

try
{
    $releaseId = $wc.UploadString($uri,"")

    $url = "$orchestratorService/ReleaseStatus?releaseId=$releaseId"

    $releaseStatus = $wc.DownloadString($url)

    Write-Host -NoNewline "`nReleasing ..."

    while($status[$releaseStatus] -eq "InProgress")
    {
        Start-Sleep -s 5
        $releaseStatus = $wc.DownloadString($url)
        Write-Host -NoNewline "."
    }

    " done.`n`nRelease completed with {0} status." -f $status[$releaseStatus]
}
catch [System.Net.WebException]
{
  if ($exitCode -eq 0) { $exitCode = 1 }
  if ($_.Exception.InnerException) {
       Write-Host $_.Exception.InnerException.Message
  } else {
       Write-Host $_.Exception.Response.Headers.GetValues("ExceptionDetails")[0]
  }
}
catch [System.Exception]
{
    if ($exitCode -eq 0) { $exitCode = 1 }
    Write-Host "`n$_`n" -ForegroundColor Red
}

if ($exitCode -eq 0)
{
  "`nThe script completed successfully.`n"
}
else
{
  $err = "Exiting with error: " + $exitCode + "`n"
  Write-Host $err -ForegroundColor Red
}

Pop-Location

exit $exitCode
